function generational_blsover(prob, seed)


rng(seed, 'twister');
eval('parameter_load'); % IMPORTANT, variables are load externally

n_FE             = 0;    % record lowerlevel FEs
n_up            = 0;    % record upperlevel FEs
prob            = eval(prob);
method       = 'generational';

llmatch_p.prob             = prob;
llmatch_p.seed             = seed;
llmatch_p.method        = method;


%------------------Process starts--------------------
%---xu initialization
xu = lhsdesign(inisize_u,prob.n_uvar,'criterion','maximin','iterations',1000);
xu = repmat(prob.xu_bl, inisize_u, 1) + repmat((prob.xu_bu - prob.xu_bl), inisize_u, 1) .* xu;

xl	= [];
%---xu match its xl and evaluate fu
for i = 1:inisize_u
    fprintf('Initialition xu matching process iteration %d\n', i);
    [xl(i, :), n, flag]     = genernational_llmatch(xu(i, :), llmatch_p, visual, noSQP_search);
    n_FE                   = n_FE + n;    
end

%---xu evaluation
[fu, fc]                = prob.evaluate_u(xu, xl);
arc_xu = xu; arc_xl = xl; arc_fu = fu; arc_cu = fc;
param.GPR_type = 2;
param.no_trials   = 1;

% prepare EA parameter
param.gen            = generational_gen;
param.popsize      = inisize_u;

for iter = 1: generational_iteru
    
    fu_norm           = normalization_z(fu);
    krg_obj             = Train_GPR(xu , fu_norm, param);
    krg_con            = {};
    
    % prepare predictor
    arc_obj.x               = xu;
    arc_obj.muf           = fu_norm;
    
    % KB objective function
    funh_obj              = @(x)Predict_GPR(krg_obj, x, param, arc_obj);
    funh_con             = @(x)NoConstraint(x);
    
    [~,~,~, archive]    = gsolver(funh_obj, prob.n_uvar,  prob.xu_bl, prob.xu_bu, [], funh_con, param);
    
    
    % evaluate last generation
    new_xu   = archive.pop_last.X;
    new_xl    =  [];
    for ii = 1: size(new_xu, 1)
         [new_xl(ii, :), n, ~] = genernational_llmatch(new_xu(i, :), llmatch_p, visual, noSQP_search);
         n_FE                     = n_FE + n;    
    end
    
    [new_fu, new_fc]  = prob.evaluate_u(new_xu, new_xl);
    
    %  add into archive 
    for ii = 1: size(new_xu, 1)        
          [arc_xu, arc_xl, arc_fu, arc_cu]...
                          = add_entry(arc_xu, arc_xl, arc_fu, arc_cu, new_xu(ii, :), new_xl(ii, :), new_fu(ii, :), []);
          [xu, xl, fu, fc]...
                          = add_entry(xu, xl,fu, fc, new_xu(ii, :), new_xl(ii, :), new_fu(ii, :), []);
          [xu, xl, fu, fc]      = keepdistance_upper(xu, fu, fc, xl, prob.xu_bu, prob.xu_bl);  
    end
        
end

% --- post process
if coresteps  % no upper local search
    n_up   =  size(arc_xu, 1);
    n_low  =  n_FE;
    ulego_coreending(arc_xu, arc_fu, arc_cu, arc_xl, prob, seed, n_up, n_low, method);
else
    
    if blsearch % upper bilevel local search
        [xu_start, ~, ~, ~, ~]       =  localsolver_startselection(arc_xu,  arc_fu, arc_cu);
        nest_parameter.arc_con       =  arc_cu;  % for whether problem has contraints or not
        if  nest_parameter.global_search
            nest_parameter.xlinit    =  [];else
            nest_parameter.xlinit    =  prob.xl_bl + rand(1, prob.n_lvar).* (prob.xl_bu - prob.xl_bl);
        end
        [xu_new, xl_new, nu, nl]...
            =  bl_localsearch(prob, xu_start, nest_parameter);
        [fu_new, cu_new]             =  prob.evaluate_u(xu_new, xl_new);
        
        n_up                         = n_up + nu;
        n_FE                         = n_FE + nl;
        [arc_xu, arc_xl, arc_fu, arc_cu]...
                                     = add_entry(arc_xu, arc_xl, arc_fu, arc_cu,xu_new, xl_new, fu_new, cu_new);
        
        
    end
    % re-evaluate
    [xu_final, ~, ~, ~, index]   = localsolver_startselection(arc_xu,  arc_fu, arc_cu);  
    re_evalparameter.xlinit      = arc_xl(index, :);
    [xl_final, nl]               = llmatch_trueEval(xu_final, prob, re_evalparameter);
    [fu_final, cu_final]         = prob.evaluate_u(xu_final, xl_final);
    
    % --- archive add entry
    [arc_xu, arc_xl, arc_fu, arc_cu]...
                                 = add_entry(arc_xu, arc_xl, arc_fu, arc_cu,xu_final, xl_final, fu_final, cu_final);
    n_FE                         = n_FE + nl;
    n_up                         = n_up + size(arc_xu, 1);
    
    
    ulego_coreending(arc_xu, arc_fu, arc_cu, arc_xl, prob, seed, n_up, n_FE, method);
end


end


function f = NoConstraint(x)
f = [];
end


function [x1, x2, f, c] = add_entry(x1, x2, f, c, x1_new, x2_new, f_new, c_new)
x1 = [x1; x1_new]; 
x2 = [x2; x2_new]; 
f  = [f;  f_new]; 
c  = [c;  c_new];
end