function [match_xl, n_fev, flag] = genernational_llmatch(xu, llmatch_p, varargin)

visualization      = varargin{1};
noSQP_search    = varargin{2};
prob                 = llmatch_p.prob;
num_gen          = llmatch_p.generational_gen;
init_size             = llmatch_p.egoinitsize;
max_iter           =  llmatch_p.max_iterL;

l_nvar                   = prob.n_lvar;
param.GPR_type   = 2;
param.no_trials     = 1;

xu_init         = repmat(xu, init_size, 1);
arc_x           = lhsdesign(init_size,l_nvar,'criterion','maximin','iterations',1000);
arc_x           = repmat( prob.xl_bl, init_size, 1) ...
                    + repmat(( prob.xl_bu - prob.xl_bl), init_size, 1) .* arc_x;
[arc_f, arc_c]   = prob.evaluate_l(xu_init, arc_x);

for iter = 1: max_iter
    
     % update surrogate
    arc_f_norm             = normalization_z(arc_f);
    krg_obj                 = Train_GPR(arc_x, arc_f_norm, param);
    krg_con                = {};
    
    % prepare predictor
    arc_obj.x               = arc_x;
    arc_obj.muf           = arc_f_norm;
    
    % KB objective function
    funh_obj              = @(x)Predict_GPR(krg_obj, x, param, arc_obj);
    funh_con             = @(x)NoConstraint(x);

    % prepare EA parameter
    param.gen            = num_gen;
    param.popsize        = init_size;
 
    [~,~,~, archive]    = gsolver(funh_obj, l_nvar,  prob.xl_bl, prob.xl_bu, [], funh_con, param);
    
    % evaluate last generation
    new_xl   = archive.pop_last.X;
    new_xu  = repmat(xu, size(new_xl, 1), 1);
    new_fl    = prob.evaluate_l(new_xu, new_xl);
    
    % update archive 
    for ii = 1: size(new_xl, 1)
         arc_x                     = [arc_x; new_xl(ii, : )];
         arc_f                     = [arc_f; new_fl(ii, :)];
         arc_c                     = [];
         [arc_x, arc_f,  arc_c,]...
                                        = keepdistance(arc_x, arc_f,  arc_c, prob.xl_bu, prob.xl_bl);
    end
end


[best_x, best_f, best_c, s] =  localsolver_startselection(arc_x, arc_f, arc_c);

if noSQP_search
    match_xl = best_x;
    n_fev    = init_size * (max_iter + 1);
    flag     = s;
else
    if size(arc_f, 2)> 1
        error('local search does not apply to MO');
    end
    [match_xl, flag, num_eval] = ll_localsearch(best_x, best_f, best_c, s, xu, prob);
    n_global                              = (max_iter + 1) * init_size;
    n_fev                                 = n_global + num_eval;       % one in a population is evaluated
end


end


function f = NoConstraint(x)
f = [];
end