classdef smd5x
    properties
        p = 1;
        q = 1;
        r = 1;
        n_lvar;
        n_uvar;
        xu_bl;
        xu_bu;
        xl_bl;
        xl_bu;
        name;
        uopt = [];
        lopt = [];
        xu_prime = [];
        xl_prime = [];
    end
    methods
        function obj = smd5x(p, q, r)
            if nargin == 3
                obj.p = p;
                obj.q = q;
                obj.r = r;
            end
            obj.name = 'SMD5_tp6';
            
            % level variables
            obj.n_lvar = obj.q + obj.r;
            obj.n_uvar = obj.p + obj.r;
            
            % bounds
            %init bound upper level
            xu_bl_1 = ones(1, obj.p)* (-5.0);
            xu_bu_1 = ones(1, obj.p) * 10.0;
            xu_bl_2 = ones(1, obj.r)* (-5.0);
            xu_bu_2 = ones(1, obj.r) * 10.0;
            obj.xu_bl = [xu_bl_1, xu_bl_2];
            obj.xu_bu = [xu_bu_1, xu_bu_2];
            
            % init bound lower level
            xl_bl_1 = ones(1, obj.q) * (-5.0);
            xl_bu_1 = ones(1, obj.q) * 10.0;
            xl_bl_2 = ones(1, obj.r) * (-5.0);
            xl_bu_2 = ones(1, obj.r) * 10.0;
            obj.xl_bl = [xl_bl_1, xl_bl_2];
            obj.xl_bu = [xl_bu_1, xl_bu_2];
            
        end
        
        function [f, c] = evaluate_u(obj, xu, xl)
            xu1 = xu(:, 1 : obj.p);
            xu2 = xu(:, obj.p + 1: obj.p + obj.r);
            
            xl1 = xl(:, 1 : obj.q);
            xl2 = xl(:, obj.q+1 : obj.q+obj.r);
            
            term2 = tp6(xl1); 
            %-obj
            f = sum((xu1).^2, 2) ...
                - term2 ...
                + sum((xu2).^2, 2) - sum((abs(xu2) - xl2.^2).^2, 2);
            %-cie
            c = [];
        end
        function [f, c] = evaluate_l(obj, xu, xl)
            xu1 = xu(:, 1 : obj.p);
            xu2 = xu(:, obj.p + 1: obj.p + obj.r);
            
            xl1 = xl(:, 1 : obj.q);
            xl2 = xl(:, obj.q+1 : obj.q+obj.r);
            %-obj
            term2 = tp6(xl1); 
            f = sum((xu1).^2, 2) ...
                + term2 ...
                + sum((abs(xu2) - xl2.^2).^2, 2);
            %-cie
            c = [];
            
            
        end
		
		function f = tp6(obj, xl)
			%-obj
            f = 0;
            n = 1;
           for i = 1: obj. q
               fi = 2 * sin(10 * exp(-0.2 * xl(:, i)) .* xl(:, i)) .* exp(-0.25 * xl(:, i));
               f = f+fi;
           end
           
           f = f ./ n;
		end
        
        function xl_prime = get_xlprime(obj, xu)
            xl1 = ones(1, obj.q);
            xu2 = xu(1, obj.p + 1:end);
            xl2 = sqrt(abs(xu2));
            xl = [xl1, xl2];
            xl_prime = xl;
        end
    end
end
